@@ -22,6 +22,7 @@
 import os, glob, socket, shutil
 import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.Dynagen.pemu_lib as pix
+import GNS3.Dynagen.simhost_lib as lwip
 import GNS3.Globals as globals
 from PyQt4 import QtGui, QtCore
 from GNS3.Utils import translate, debug
@@ -35,6 +36,10 @@ from GNS3.Node.ETHSW import ETHSW, init_ethsw_id
 from GNS3.Node.FRSW import FRSW, init_frsw_id
 from GNS3.Node.Cloud import Cloud, init_cloud_id
 from GNS3.Node.FW import FW, init_fw_id
+from GNS3.Node.SIMHOST import SIMHOST, init_simhost_id
+from GNS3.SimhostManager import SimhostManager
+from GNS3.Node.AbstractNode import AbstractNode
+from GNS3.Annotation import Annotation
 
 class Topology(QtGui.QGraphicsScene):
     """ Topology class
@@ -50,8 +55,30 @@ class Topology(QtGui.QGraphicsScene):
         self.dynagen = globals.GApp.dynagen
         self.changed = False
 
+        width = globals.GApp.systconf['general'].scene_width
+        height = globals.GApp.systconf['general'].scene_height
+        
         QtGui.QGraphicsScene.__init__(self, parent)
-        self.setSceneRect(-1000, -500, 2000, 1000)
+        self.setSceneRect(-(width / 2), -(height / 2), width, height)
+
+    def mousePressEvent(self, event):
+
+        srcnode = globals.GApp.scene.getSourceNode()
+        item = self.itemAt(event.scenePos())
+        if item and ((isinstance(item, AbstractNode) and \
+        globals.currentLinkType == globals.Enum.LinkType.Manual) or \
+        isinstance(srcnode, FW) or isinstance(srcnode, FRSW) or \
+        isinstance(srcnode, ATMBR) or isinstance(srcnode, ATMSW) or isinstance(srcnode, Cloud)):
+            # In few circumstances, QtGui.QGraphicsScene.mousePressEvent()
+            # send the mousePressEvent to the wrong item; we need to
+            # correct this behaviour for 'Manual Link' mode. We force the 
+            # event to be send to the right item and activate workaround
+            # so that the false recipiend ignore it.
+            item.mousePressEvent(event)
+            globals.workaround_ManualLink = True
+        elif item and isinstance(item, Annotation):
+            item.mousePressEvent(event)
+        QtGui.QGraphicsScene.mousePressEvent(self, event)
 
     def cleanDynagen(self):
         """ Clean all dynagen data
@@ -64,6 +91,8 @@ class Topology(QtGui.QGraphicsScene):
                 continue
         if globals.GApp.HypervisorManager:
             globals.GApp.HypervisorManager.stopProcHypervisors()
+        if globals.GApp.SimhostManager:
+            globals.GApp.SimhostManager.stopProcHypervisors()
         if globals.GApp.PemuManager:
             globals.GApp.PemuManager.stopPemu()
         self.dynagen.dynamips.clear()
@@ -97,6 +126,7 @@ class Topology(QtGui.QGraphicsScene):
         init_fw_id()
         init_cloud_id()
         init_decoration_id()
+        init_simhost_id()
         self.cleanDynagen()
 
     def getNode(self, id):
@@ -216,7 +246,7 @@ class Topology(QtGui.QGraphicsScene):
             node.set_ghostios(True)
 
     def firewallSetup(self, node):
-        """ Start a connetion to Pemu & set defaults
+        """ Start a connection to Pemu & set defaults
         """
 
         if globals.GApp.systconf['pemu'].enable_PemuManager:
@@ -276,6 +306,45 @@ class Topology(QtGui.QGraphicsScene):
         node.set_string_option('serial', globals.GApp.systconf['pemu'].default_pix_serial)
         return True
 
+    def simhostSetup(self, node):
+        """ Start a connection to simhost hypervisor
+        """
+
+        hypervisor = globals.GApp.SimhostManager.allocateHypervisor()
+        if hypervisor == None:
+            return
+            
+        host = 'localhost'
+        port = hypervisor['port']
+        simhost_name = host + ':' + str(port)
+        debug('Simhost hypervisor on : ' + simhost_name)
+        
+        if not self.dynagen.dynamips.has_key(simhost_name):
+            #create the Simhost hypervisor instance and add it to global dictionary
+            self.dynagen.dynamips[simhost_name] = lwip.LWIP(host, port, globals.simhost_hypervisor_baseudp)
+            self.dynagen.dynamips[simhost_name].reset()
+            
+            self.dynagen.get_defaults_config()
+            self.dynagen.update_running_config()
+            self.dynagen.dynamips[simhost_name].configchange = True
+            
+            if globals.GApp.workspace.projectWorkdir:
+                workdir = globals.GApp.workspace.projectWorkdir
+            elif globals.GApp.systconf['simhost'].workdir:
+                workdir = globals.GApp.systconf['simhost'].workdir
+            else:
+                realpath = os.path.realpath(self.dynagen.global_filename)
+                workdir = os.path.dirname(realpath)
+            try:
+                self.dynagen.dynamips[simhost_name].workingdir = workdir
+            except lib.DynamipsError, msg:
+                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "Simhost error"),  unicode(workdir + ': ') + unicode(msg))
+                del self.dynagen.dynamips[simhost_name]
+                return False
+
+        node.set_hypervisor(self.dynagen.dynamips[simhost_name])
+        return True
+        
     def addNode(self, node):
         """ Add node in the topology
             node: object
@@ -309,7 +378,7 @@ class Topology(QtGui.QGraphicsScene):
                         (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "IOS image"),
                                                                       translate("Topology", "Please choose an image"), selected_images, 0, False)
                         if ok:
-                            image_to_use = str(selection)
+                            image_to_use = unicode(selection)
                         else:
                             return
                 else:
@@ -342,6 +411,9 @@ class Topology(QtGui.QGraphicsScene):
                     return
                 if self.firewallSetup(node) == False:
                     return
+                    
+            if isinstance(node, SIMHOST) and self.simhostSetup(node) == False:
+                return
 
             QtCore.QObject.connect(node, QtCore.SIGNAL("Add link"), globals.GApp.scene.slotAddLink)
             QtCore.QObject.connect(node, QtCore.SIGNAL("Delete link"), globals.GApp.scene.slotDeleteLink)
@@ -406,14 +478,43 @@ class Topology(QtGui.QGraphicsScene):
     def recordLink(self, srcid, srcif, dstid, dstif, src_node, dest_node):
         """ Record the link in the topology
         """
+
+        multi = 0
+        d1 = 0
+        d2 = 1
+        edges = src_node.getEdgeList()
+        for edge in edges:
+            if edge.dest.hostname == dest_node.hostname:
+                d1 += 1
+            if edge.source.hostname == dest_node.hostname:
+                d2 += 1
+        
+        if len(edges) > 0:
+            if d2 - d1 == 2:
+                srcid, dstid = dstid, srcid
+                srcif, dstif = dstif, srcif
+                src_node, dest_node = dest_node, src_node
+                multi = d1 + 1
+            elif d1 >= d2:
+                srcid, dstid = dstid, srcid
+                srcif, dstif = dstif, srcif
+                src_node, dest_node = dest_node, src_node
+                multi = d2
+            else:
+                multi = d1
+
+        # MAX 7 links on the scene between 2 nodes
+        if multi > 3:
+            multi = 0
+        
         if (globals.currentLinkType == globals.Enum.LinkType.Serial or globals.currentLinkType == globals.Enum.LinkType.ATM) or \
             (globals.currentLinkType == globals.Enum.LinkType.Manual and ((srcif[0] == 's' or srcif[0] == 'a' or dstif[0] == 's' or dstif[0] == 'a') or \
             (isinstance(src_node, ATMSW) or isinstance(src_node, FRSW) or isinstance(dest_node, ATMSW) or isinstance(dest_node, FRSW)))):
             # interface is serial or ATM
-            link = Serial(self.__nodes[srcid], srcif, self.__nodes[dstid], dstif)
+            link = Serial(self.__nodes[srcid], srcif, self.__nodes[dstid], dstif, Multi=multi)
         else:
             # by default use an ethernet link
-            link = Ethernet(self.__nodes[srcid], srcif, self.__nodes[dstid], dstif)
+            link = Ethernet(self.__nodes[srcid], srcif, self.__nodes[dstid], dstif, Multi=multi)
 
         self.__links.add(link)
         self.addItem(link)
@@ -458,17 +559,18 @@ class Topology(QtGui.QGraphicsScene):
         elif not isinstance(src_node, IOSRouter) and not isinstance(dst_node, IOSRouter):
             if (isinstance(src_node, ETHSW) and not type(dst_node) in (IOSRouter, Cloud, FW)) or (isinstance(dst_node, ETHSW) and not type(src_node) in (IOSRouter, Cloud, FW)) \
                 or (type(src_node) in (ATMSW, FRSW, ATMBR) and not isinstance(dst_node, IOSRouter)) or (type(dst_node) in (ATMSW, FRSW, ATMBR) and not isinstance(src_node, IOSRouter)) \
-                or (isinstance(src_node, FW) and isinstance(dst_node, Cloud)) or (isinstance(dst_node, FW) and isinstance(src_node, Cloud)):
+                or (isinstance(src_node, FW) and isinstance(dst_node, Cloud)) or (isinstance(dst_node, FW) and isinstance(src_node, Cloud)) \
+                or (isinstance(src_node, SIMHOST) and isinstance(dst_node, Cloud)) or (isinstance(dst_node, SIMHOST) and isinstance(src_node, Cloud)):
                 QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "Connection"),  translate("Topology", "Can't connect these devices"))
                 return False
-            elif (isinstance(dst_node, Cloud) or isinstance(dst_node,FW)) and isinstance(src_node, ETHSW):
+            elif (isinstance(dst_node, Cloud) or isinstance(dst_node,FW) or isinstance(dst_node,SIMHOST)) and isinstance(src_node, ETHSW):
                 if not src_node.hypervisor:
                     debug('Allocate a hypervisor for ethsw ' + src_node.hostname)
                     if globals.GApp.HypervisorManager and not globals.GApp.HypervisorManager.allocateHypervisor(src_node):
                         QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "Connection"),  translate("Topology", "You have to connect at least one router to the switch"))
                         return False
                     src_node.get_dynagen_device()
-            elif (isinstance(src_node, Cloud) or isinstance(src_node, FW)) and isinstance(dst_node, ETHSW):
+            elif (isinstance(src_node, Cloud) or isinstance(src_node, FW) or isinstance(src_node, SIMHOST)) and isinstance(dst_node, ETHSW):
                 if not dst_node.hypervisor:
                     debug('Allocate a hypervisor for ethsw ' + dst_node.hostname)
                     if globals.GApp.HypervisorManager and not globals.GApp.HypervisorManager.allocateHypervisor(dst_node):
@@ -476,15 +578,15 @@ class Topology(QtGui.QGraphicsScene):
                         return False
                     dst_node.get_dynagen_device()
         else:
-            if not isinstance(src_node, IOSRouter) and not isinstance(src_node, Cloud) and not isinstance(src_node, FW) and not src_node.hypervisor:
+            if not isinstance(src_node, IOSRouter) and not isinstance(src_node, Cloud) and not isinstance(src_node, FW) and not isinstance(src_node, SIMHOST) and not src_node.hypervisor:
                 debug('Set hypervisor ' + dst_node.hypervisor.host + ':' + str(dst_node.hypervisor.port) + ' to ' + src_node.hostname)
                 src_node.set_hypervisor(dst_node.hypervisor)
-            elif not isinstance(dst_node, IOSRouter) and not isinstance(dst_node, Cloud) and not isinstance(dst_node, FW) and not dst_node.hypervisor:
+            elif not isinstance(dst_node, IOSRouter) and not isinstance(dst_node, Cloud) and not isinstance(dst_node, FW) and not isinstance(dst_node, SIMHOST) and not dst_node.hypervisor:
                 debug('Set hypervisor ' + src_node.hypervisor.host + ':' + str(src_node.hypervisor.port) + ' to ' + dst_node.hostname)
                 dst_node.set_hypervisor(src_node.hypervisor)
 
         try:
-            if isinstance(src_node, IOSRouter) or isinstance(src_node, FW):
+            if isinstance(src_node, IOSRouter) or isinstance(src_node, FW) or isinstance(src_node, SIMHOST) :
                 srcdev = self.__nodes[srcid].get_dynagen_device()
                 if type(dst_node) == Cloud:
                     debug('Connect link from ' + srcdev.name + ' ' + srcif +' to ' + dstif)
@@ -493,7 +595,7 @@ class Topology(QtGui.QGraphicsScene):
                     dstdev = dst_node.get_dynagen_device()
                     debug('Connect link from ' + srcdev.name + ' ' + srcif +' to ' + dstdev.name + ' ' + dstif)
                     self.dynagen.connect(srcdev, srcif, dstdev.name + ' ' + dstif)
-            elif isinstance(dst_node, IOSRouter) or isinstance(dst_node, FW):
+            elif isinstance(dst_node, IOSRouter) or isinstance(dst_node, FW) or isinstance(dst_node, SIMHOST):
                 dstdev = dst_node.get_dynagen_device()
                 if type(src_node) == Cloud:
                     debug('Connect link from ' + dstdev.name + ' ' + srcif +' to ' + dstif)
@@ -517,10 +619,14 @@ class Topology(QtGui.QGraphicsScene):
             if not isinstance(src_node, IOSRouter) and not isinstance(src_node, FW):
                 src_node.startNode()
             elif src_node.state == 'running':
+                if isinstance(src_node, SIMHOST):
+                    src_node.configureInterfaces()
                 src_node.startupInterfaces()
             if not isinstance(dst_node, IOSRouter) and not isinstance(dst_node, FW):
                 dst_node.startNode()
             elif dst_node.state == 'running':
+                if isinstance(dst_node, SIMHOST):
+                    dst_node.configureInterfaces()
                 dst_node.startupInterfaces()
         except lib.DynamipsError, msg:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "Dynamips error"),  unicode(msg))
@@ -529,6 +635,7 @@ class Topology(QtGui.QGraphicsScene):
         globals.GApp.mainWindow.treeWidget_TopologySummary.refresh()
         self.dynagen.update_running_config()
         self.changed = True
+
         return True
 
     def deleteLink(self, link):
